# bambot/templates/agent_template.py
import os

class Agent:
    def __init__(self, data=None):
        # Load environment variables
        self.env_vars = {key: value for key, value in os.environ.items()}

        # Initialize agent-specific configurations
        self.setup(data)

    def setup(self, data=None):
        # Validate input data
        if data is not None:
            self.data = data
        else:
            self.data = {}

        # Implement agent-specific setup logic here
        # You can use self.data and self.env_vars in this method

    def run(self, data=None):
        print(f"Running the {self.__class__.__name__} agent...")

        # Validate input data
        if data is not None:
            self.data = data

        # Add agent-specific logic here
        # You can use self.data and self.env_vars in this method

        # Return a sample response
        return {"message": "Hello from the agent!"}


# bambot/docker_utils.py
import os
import docker
from tqdm import tqdm

def build_image(container_dir):
    """Build a Docker image for an AI agent"""
    docker_client = docker.from_env()
    image, _ = docker_client.images.build(path=container_dir, tag=f"{os.path.basename(container_dir)}:latest", dockerfile="Dockerfile")
    return image

def create_docker_container(container_name, project_dir, env_vars=None):
    """Create a Docker container for an AI agent"""
    docker_client = docker.from_env()
    container = docker_client.containers.create(
        f"{container_name}:latest",
        name=container_name,
        environment=env_vars,
        detach=True,
        tty=True,
        stdin_open=True,
        ports={'1337/tcp': ('127.0.0.1', 1337)},
        volumes={project_dir: {'bind': '/app', 'mode': 'rw'}},
    )
    return container

def prune_system(progress_bar=None):
    """Prune Docker system resources"""
    docker_client = docker.from_env()
    tasks = [
        docker_client.containers.prune,
        docker_client.images.prune,
        docker_client.networks.prune,
        docker_client.volumes.prune,
    ]
    if progress_bar:
        for task in tasks:
            task()
            progress_bar.update(1)
    else:
        with tqdm(total=4, desc="Pruning system resources") as pbar:
            for task in tasks:
                task()
                pbar.update(1)

def list_apps(prefix=""):
    """List all Bam apps"""
    docker_client = docker.from_env()
    containers = docker_client.containers.list(all=True)
    if prefix:
        containers = [c for c in containers if c.name.startswith(prefix)]
    return containers


# bambot/project.py
import os
import pkg_resources
from .utils import generate_app_name, echo_info, echo_success, echo_warning

TEMPLATE_FILES = [
    "agent_template.py", "Dockerfile", "agent_runner.py", "requirements.txt", "server.py"
]

def init_project(container_name=None, agent_type="langchain"):
    """Initialize a new Bam project"""
    if not container_name:
        container_name = generate_app_name()

    project_dir = os.path.join(os.getcwd(), container_name)
    if os.path.exists(project_dir):
        echo_warning(f"Directory '{container_name}' already exists. Skipping project initialization.")
        return

    echo_info(f"Initializing project '{container_name}'...")

    # Create project directory
    os.makedirs(project_dir, exist_ok=True)

    # Copy template files to the project directory
    templates_dir = pkg_resources.resource_filename(__name__, 'templates')
    for filename in TEMPLATE_FILES:
        template_path = os.path.join(templates_dir, filename)
        with open(template_path, "r") as f:
            template = f.read()
        if filename == "agent_runner.py":
            template = template.replace("{{AGENT_TYPE}}", agent_type)
        file_path = os.path.join(project_dir, filename.replace("agent_template.py", f"{agent_type}_agent.py"))
        with open(file_path, "w") as f:
            f.write(template)

    # Create .env file
    env_file_path = os.path.join(project_dir, ".env")
    with open(env_file_path, "w") as f:
        f.write("OPENAI_API_KEY=your_openai_api_key\n")

    echo_success(f"Project '{container_name}' initialized successfully!")


# bambot/app.py
import os
from flask import Flask
from flask_reloader import run_with_reloader
from agent_runner import Agent

agent = Agent()

app = Flask(__name__)
app.config.from_envvar('APP_CONFIG_FILE')

@app.route("/")
def index():
    return "Hello, World!"

@app.route("/api/agent", methods=["POST"])
def run_agent():
    # Your agent logic here
    pass

if __name__ == "__main__":
    run_with_reloader(app, extra_files=[os.path.join(os.getcwd(), 'app.py')])


# bambot/build.py
import os
import docker
from .utils import echo_info, echo_success, echo_error, contextmanager_spinner


def build_project():
    """Build the Bam project"""
    project_dir = os.getcwd()
    container_name = os.path.basename(project_dir)

    with contextmanager_spinner(f"Building Docker image for '{container_name}'...") as spinner:
        try:
            docker_client = docker.from_env()
            response = docker_client.api.build(
                path=project_dir, 
                tag=f"{os.path.basename(project_dir)}:latest", 
                dockerfile="Dockerfile", 
                rm=True,
                decode=True
            )

            # Process and print build logs as they arrive
            for output in response:
                if 'stream' in output:
                    print(output['stream'].strip())
                if 'error' in output:
                    echo_error(f"Error during build: {output['errorDetail']['message']}")
                    break
            spinner.stop()
            echo_success("Docker image built successfully!")

            # Create the container and mount the project directory as a volume
            container = docker_client.containers.create(
                f"{container_name}:latest",
                name=container_name,
                tty=True,
                stdin_open=True,
                ports={'1337/tcp': ('127.0.0.1', 1337)},
                volumes={project_dir: {'bind': '/app', 'mode': 'rw'}}
            )
            echo_success(f"Container '{container_name}' created successfully!")
        except docker.errors.BuildError as e:
            spinner.stop()
            echo_error(f"Error building Docker image: {e}")
            raise

    echo_success(f"Project '{container_name}' built successfully! You can now start the AI agent by running 'bam start'.")


# bambot/templates/langchain_template.py
from langchain import OpenAI, ConversationChain

class Agent:
    def __init__(self):
        self.llm = OpenAI(temperature=0)
        self.conversation = ConversationChain(llm=self.llm)

    def run(self):
        print("Running the Langchain agent...")
        while True:
            user_input = input("User: ")
            if user_input.lower() == 'exit':
                break
            response = self.conversation.predict(input=user_input)
            print("Agent:", response)


# bambot/templates/server.py
import logging
from colorama import Fore, Style
from flask import Flask, render_template_string, jsonify, request
from flask_cors import CORS
import datetime
from dotenv import load_dotenv
load_dotenv()

from agent_runner import Agent

# Custom logging formatter for color-coded logs
class ColoredFormatter(logging.Formatter):
    def __init__(self, msg, use_color=True):
        super().__init__(msg)
        self.use_color = use_color

    def format(self, record):
        level_color = {
            "DEBUG": Fore.CYAN,
            "INFO": Fore.GREEN,
            "WARNING": Fore.YELLOW,
            "ERROR": Fore.RED,
            "CRITICAL": Fore.RED + Style.BRIGHT
        }

        timestamp = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        level_name = record.levelname
        message = record.getMessage()

        if self.use_color and level_name in level_color:
            level_name = level_color[level_name] + level_name + Style.RESET_ALL
            message = level_color[record.levelname] + message + Style.RESET_ALL

        log_line = f"{Fore.MAGENTA}[{timestamp}]{Style.RESET_ALL} {Fore.BLUE}[{level_name}]{Style.RESET_ALL} {message}"
        return log_line

# Setup Flask application
app = Flask(__name__)
app.name = "Bam AI Agent"
CORS(app)  # Enable CORS for all routes
agent = Agent()

# Logger setup
logger = logging.getLogger("bam_logger")
logger.setLevel(logging.DEBUG)
console_handler = logging.StreamHandler()
console_handler.setFormatter(ColoredFormatter("[%(levelname)s] %(message)s"))
logger.addHandler(console_handler)

# Routes
@app.route("/")
def index():
    """Serve the landing page with helpful links and commands."""
    template = """
    <!DOCTYPE html>
    <html>
    <head>
        <title>Bam AI Agent</title>
        <style>
            body { font-family: Arial, sans-serif; text-align: center; padding: 20px; }
            h1 { color: #333; display: flex; align-items: center; justify-content: center; }
            p { margin-bottom: 10px; }
            code { color: #007bff; font-weight: bold; }
        </style>
    </head>
    <body>
        <h1>🚀 Bam AI Agent</h1>
        <p>To get started, explore the following endpoints:</p>
        <p><a href="/api/agent/info">Agent Info</a></p>
        <p><a href="/api/agent/health">Agent Health</a></p>
        <p><code>curl -X POST -d '{"input": "whats up?"}' localhost:1337/api/agent</code></p>
    </body>
    </html>
    """
    return render_template_string(template)

@app.route("/api/agent", methods=["POST"])
def run_agent():
    """Run the AI agent and return its response."""
    input_data = request.get_json(force=True)
    response = agent.run(input_data)
    logger.info(f"Agent response: {response}")
    return jsonify(response)

@app.route("/api/agent/info")
def agent_info():
    """Return information about the AI agent."""
    info = {"name": agent.__class__.__name__, "description": "AI agent powered by Bam framework"}
    logger.info(f"Agent info: {info}")
    return jsonify(info)

@app.route("/api/agent/health")
def agent_health():
    """Check the health of the AI agent."""
    health = {"status": "ok"}
    logger.info(f"Agent health: {health}")
    return jsonify(health)

@app.errorhandler(404)
def not_found(error):
    """Serve a custom 404 error page."""
    template = """
    <!DOCTYPE html>
    <html>
    <head>
        <title>404 - Page Not Found</title>
        <style>
            body { font-family: Arial, sans-serif; text-align: center; padding: 20px; }
            h1 { color: #333; }
        </style>
    </head>
    <body>
        <h1>404 - Page Not Found</h1>
        <p>The requested page could not be found.</p>
        <p><a href="/">Go back to the homepage</a></p>
    </body>
    </html>
    """
    return render_template_string(template), 404

if __name__ == "__main__":
    app.run(host="0.0.0.0", port=1337, debug=True)


# bambot/cli.py
import click
from .project import init_project
from .build import build_project
from .run import run_project
from .utils import setup_logging, echo_info, echo_success, echo_error, confirm_with_style, prune_system

@click.group()
def cli():
    """Bam CLI application"""
    setup_logging()

@cli.command("init")
@click.argument("container_name", required=False)
def init_command(container_name):
    """Initialize a new Bam project"""
    try:
        init_project(container_name)
    except Exception as e:
        echo_error(f"Error initializing project: {e}")

@cli.command("build")
def build_command():
    """Build the Bam project"""
    if confirm_with_style("Do you want to clean up unused Docker resources before building?"):
        prune_system()
        echo_success("Unused Docker resources cleaned up successfully!")

    try:
        build_project()
    except Exception as e:
        echo_error(f"Error building project: {e}")

@cli.command("start")
@click.option("--container-name", "-n", default=None, help="Name of the container to start")
def start_command(container_name):
    """Start the Bam project"""
    try:
        run_project(container_name)
    except Exception as e:
        echo_error(f"Error starting project: {e}")

def main():
    cli()

if __name__ == "__main__":
    main()


# bambot/utils.py
import random
import string
import logging
from colorama import Fore, Style
from halo import Halo
import docker
from contextlib import contextmanager


def generate_app_name():
    # Simplified structure with less verbosity
    prefix = "bambot"
    # Combine adjectives and nouns for more combinations with fewer parts
    descriptors = ["cool", "smart", "super", "zap", "zip", "jet", "jam", "pod"]
    # Generate a two-digit number to reduce length but maintain uniqueness
    number = random.choices(string.digits, k=2)
    return f"{prefix}-{random.choice(descriptors)}-{'' .join(number)}"

def setup_logging():
    """Set up colored logging for better visual feedback"""
    logger = logging.getLogger("bam_logger")
    logger.setLevel(logging.DEBUG)
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(ColoredFormatter("[%(levelname)s] %(message)s"))
    logger.addHandler(console_handler)

class ColoredFormatter(logging.Formatter):
    def __init__(self, msg, use_color=True):
        super().__init__(msg)
        self.use_color = use_color

    def format(self, record):
        level_color = {
            "DEBUG": Fore.CYAN,
            "INFO": Fore.GREEN,
            "WARNING": Fore.YELLOW,
            "ERROR": Fore.RED,
            "CRITICAL": Fore.RED + Style.BRIGHT
        }
        if self.use_color and record.levelname in level_color:
            record.msg = level_color[record.levelname] + record.msg + Style.RESET_ALL
        return super().format(record)

def echo_info(message, spinner=None):
    """Print an informational message with a spinner"""
    if spinner:
        spinner.start(text=message)
    else:
        print(Fore.GREEN + "[*] " + str(message) + Style.RESET_ALL)

def echo_success(message):
    """Print a success message"""
    print(Fore.GREEN + "[✓] " + str(message) + Style.RESET_ALL)

def echo_warning(message):
    """Print a warning message"""
    print(Fore.YELLOW + "[!] " + str(message) + Style.RESET_ALL)

def echo_error(message):
    """Print an error message"""
    print(Fore.RED + "[✗] " + str(message) + Style.RESET_ALL)

@contextmanager
def contextmanager_spinner(message):
    """Context manager for displaying a spinner"""
    spinner = Halo(text=message, spinner="dots")
    spinner.start()
    try:
        yield spinner
    finally:
        spinner.stop()

def confirm_with_style(message, default=False):
    """Prompt the user for confirmation with a styled message"""
    prompt = f"{Fore.YELLOW}[?]{Style.RESET_ALL} {message} "
    if default:
        prompt += f"[{Fore.GREEN}Y{Style.RESET_ALL}/{Fore.RED}n{Style.RESET_ALL}]"
    else:
        prompt += f"[{Fore.RED}y{Style.RESET_ALL}/{Fore.GREEN}N{Style.RESET_ALL}]"

    while True:
        user_input = input(f"{prompt}? ").strip().lower()
        if user_input in ("y", "yes"):
            return True
        elif user_input in ("n", "no"):
            return False
        elif not user_input and default is not None:
            return default

def prune_system():
    """Prune unused Docker resources"""
    docker_client = docker.from_env()
    docker_client.containers.prune()
    docker_client.images.prune()
    docker_client.networks.prune()
    docker_client.volumes.prune()
    echo_success("Unused Docker resources cleaned up successfully!")


# bambot/run.py
import os
import signal
import sys
import docker
from .utils import echo_info, echo_success, echo_error

def run_project(container_name=None):
    """Run the Bam project"""
    project_dir = os.getcwd()
    if not container_name:
        container_name = os.path.basename(project_dir)

    echo_info(f"Starting container: {container_name}")

    try:
        docker_client = docker.from_env()
        container = docker_client.containers.get(container_name)
    except docker.errors.NotFound:
        echo_error(f"Container '{container_name}' not found. Please run 'bam init' and 'bam build' first.")
        return

    # Start container
    container.start()

    # Set up signal handler for SIGINT (Ctrl+C)
    def signal_handler(sig, frame):
        echo_info("\nStopping container, hang in there...")
        container.stop()
        echo_success("Container gracefully stopped. You may go about your life now.")
        sys.exit(0)

    signal.signal(signal.SIGINT, signal_handler)

    # Stream container logs
    echo_info("Container logs:")
    try:
        for log in container.logs(stream=True, follow=True):
            try:
                log_str = log.decode('utf-8')
                print(log_str, end='')
            except UnicodeDecodeError:
                echo_error(f"Error decoding log line: {log}")
    except KeyboardInterrupt:
        # Handle KeyboardInterrupt separately
        signal_handler(signal.SIGINT, None)


# setup.py
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="bambot",
    version="0.4.4",
    author="Bam Corp",
    author_email="spencer@bam.bot",
    description="Containers for AI agents",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Bam-Corp/bambot",
    packages=find_packages(),
    package_data={'bambot': ['templates/*']},
    install_requires=["click", "docker", "tqdm", "python-dotenv", "halo", "colorama"],
    entry_points={
        "console_scripts": ["bam=bambot.cli:main"]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    python_requires=">=3.6",
)